import requests



class Assurka(object):
    def __init__(self):
        self.token = None
        self.testRun = None
        self.testPlan = None
        self.testPlanParams = None
        self.testSpec = None
        self.testSpecs = []
        self.urlBase = 'https://api.assurka.io'

    def assurka_login(self, projectId, secret):
        try:
            response = requests.post(self.urlBase + '/auth/api', json={
                'id': projectId, 'secret': secret
            }, headers={'Content-type': 'application/json', 'Accept': 'application/json'})
            self.token = response.json()['token']
        except requests.exceptions.HTTPError as error:
            print(error)

    def assurka_getTestPlan(self, projectId, testPlanId):
        try:
            print(self.token)
            response = requests.get(self.urlBase + '/test-plan/' + projectId + '/get/' + testPlanId,
                                    headers={'Content-type': 'application/json', 'Accept': 'application/json',
                                             'Authorization': 'Bearer ' + self.token})
            self.testPlan = response.json()
        except requests.exceptions.HTTPError as error:
            print(error)

    def assurka_getTestPlanParams(self, testPlanId):
        try:
            response = requests.get(self.urlBase + '/test-param/' + testPlanId + '/list',
                                    headers={'Content-type': 'application/json', 'Accept': 'application/json',
                                             'Authorization': 'Bearer ' + self.token})
            self.testPlanParams = response.json()
        except requests.exceptions.HTTPError as error:
            print(error)

    def assurka_createTestRun(self, testPlanId, payload):
        try:
            response = requests.post(self.urlBase + '/test-run/' + testPlanId + '/create',
                                     json=payload,
                                     headers={'Content-type': 'application/json', 'Accept': 'application/json',
                                              'Authorization': 'Bearer ' + self.token})
            self.testRun = response.json()
        except requests.exceptions.HTTPError as error:
            print(error)

    def assurka_createTestSpec(self, testRunId, name):
        try:
            response = requests.post(self.urlBase + '/test-spec/' + testRunId + '/create',
                                     json={'name': name,
                                           'testRunId': testRunId},
                                     headers={'Content-type': 'application/json', 'Accept': 'application/json',
                                              'Authorization': 'Bearer ' + self.token})
            self.testSpec = response.json()
            self.testSpecs.append(response.json())
        except requests.exceptions.HTTPError as error:
            print(error)

    def assurka_createTestCase(self, testSpecId, payload):
        try:
            response = requests.post(self.urlBase + '/test-case/' + testSpecId + '/create',
                                     json=payload,
                                     headers={'Content-type': 'application/json', 'Accept': 'application/json',
                                              'Authorization': 'Bearer ' + self.token})
        except requests.exceptions.HTTPError as error:
            print(error)

    def assurka_updateTestRun(self, testPlanId, testRunId, payload):
        try:
            response = requests.patch(self.urlBase + '/test-run/' + testPlanId + '/update/' + testRunId,
                                      json=payload,
                                      headers={'Content-type': 'application/json', 'Accept': 'application/json',
                                               'Authorization': 'Bearer ' + self.token})
            self.testRun = response.json()
        except requests.exceptions.HTTPError as error:
            print(error)

    def assurka_updateTestSpec(self, testRunId, testSpecId, payload):
        try:
            response = requests.patch(self.urlBase + '/test-spec/' + testRunId + '/update/' + testSpecId,
                                      json=payload,
                                      headers={'Content-type': 'application/json', 'Accept': 'application/json',
                                               'Authorization': 'Bearer ' + self.token})
        except requests.exceptions.HTTPError as error:
            print(error)

    def assurka_updateTestPlan(self, projectId, testPlanId, payload):
        try:
            response = requests.patch(self.urlBase + '/test-plan/' + projectId + '/update/' + testPlanId,
                                      json=payload,
                                      headers={'Content-type': 'application/json', 'Accept': 'application/json',
                                               'Authorization': 'Bearer ' + self.token})
            self.testPlan = response.json()
        except requests.exceptions.HTTPError as error:
            print(error)

    def assurka_updateTestRunCoverage(self, testPlanId, testRunId, payload):
        try:
            response = requests.patch(self.urlBase + '/test-run/' + testPlanId + '/update/' + testRunId + '/coverage',
                                      json=payload,
                                      headers={'Content-type': 'application/json', 'Accept': 'application/json',
                                               'Authorization': 'Bearer ' + self.token})

        except requests.exceptions.HTTPError as error:
            print(error)




import time

from . import Assurka


def pytest_addoption(parser):
    assurka = parser.getgroup('assurka')
    assurka.addoption("--assurka-projectId", action="store", default=None, help="assurka studio project id")
    assurka.addoption("--assurka-secret", action="store", default=None, help="assurka studio project secret id")
    assurka.addoption("--assurka-testPlanId", action="store", default=None, help="assurka studio test plan id")
    assurka.addoption("--assurka-host", action="store", default=None, help="assurka studio host url")


class AssurkaPytest:
    a = Assurka()

    def __init__(self, config):
        self.config = config
        self.projectId = None
        self.secret = None
        self.testPlanId = None
        self.testRunId = None
        self.specs = {}
        self.activate = False

    def pytest_sessionstart(self, session):
        # check to make sure we have the relevant options configured
        if hasattr(session.config.option, 'assurka_projectId'):
            self.projectId = session.config.option.assurka_projectId

        if hasattr(session.config.option, 'assurka_secret'):
            self.secret = session.config.option.assurka_secret

        if hasattr(session.config.option, 'assurka_testPlanId'):
            self.testPlanId = session.config.option.assurka_testPlanId

        if hasattr(session.config.option, 'assurka_host'):
            self.a.urlBase = session.config.option.assurka_host

        if self.secret is not None:
            self.activate = True

    def pytest_collection_modifyitems(self, session, config, items):
        if self.activate is True:
            # authenticate to assurka studio
            self.a.assurka_login(self.projectId, self.secret)
            # get the test plan and params
            self.a.assurka_getTestPlan(self.projectId, self.testPlanId)
            self.a.assurka_getTestPlanParams(self.testPlanId)

            for item in items:
                title = item.nodeid
                path = item.location[0]
                if item.obj.__doc__ is not None:
                    description = item.obj.__doc__.strip()
                else:
                    description = ''
                markers = ','.join([m.name for m in item.iter_markers()])
                self.specs.setdefault(path, []).append({
                    'path': path,
                    'title': title,
                    'description': description,
                    'numFailingTests': 0,
                    'numPassingTests': 0
                })

            # create the assurka test rum
            self.a.assurka_createTestRun(self.testPlanId, {
                'name': 'pytest run',
                'platform': 'pytest',
                'numTotalTestSuites': len(self.specs),
                'numTotalTests': len(items),
                'startTime': time.time()
            })
            self.testRunId = self.a.testRun['id']
            for spec in self.specs:
                s = self.specs[spec]
                # create the new test spec
                self.a.assurka_createTestSpec(self.a.testRun['id'], s[0]['path'])

    def pytest_runtest_logreport(self, report):
        if self.activate is True:
            if report.when == 'call':
                testSpec = next((x for x in self.a.testSpecs if x['name'] == report.location[0]), None)
                spec = self.specs[testSpec['name']]
                for item in spec:
                    if report.passed:
                        item['numPassingTests'] = item['numPassingTests'] + 1
                    else:
                        item['numFailingTests'] = item['numFailingTests'] + 1

                payload = {
                    'testRunId': self.testRunId,
                    'title': report.head_line,
                    'description': report.nodeid,
                    'status': report.outcome,
                    'duration': report.duration,
                    'testSpecId': testSpec['id']
                }
                if hasattr(report.longrepr, 'reprcrash'):
                    payload['failureMessages']: [
                        report.longreprtext
                    ]
                    payload['failureDetails'] = [
                        {
                            'lineno': report.longrepr.reprcrash.lineno,
                            'message': report.longrepr.reprcrash.message
                        }
                        ]

                self.a.assurka_createTestCase(testSpec['id'], payload)

    def pytest_sessionfinish(self, session):
        if self.activate is True:
            for testSpec in self.a.testSpecs:
                spec = self.specs[testSpec['name']][0]
                self.a.assurka_updateTestSpec(self.testRunId, testSpec['id'], {
                    'numPassingTests': spec['numPassingTests'],
                    'numFailingTests': spec['numFailingTests']
                })

            payload = {
                'numTotalTests': session.testscollected,
                'numFailedTests': session.testsfailed,
                'numPassedTests': session.testscollected - session.testsfailed
            }
            self.a.assurka_updateTestRun(self.testPlanId, self.testRunId, payload)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pytest_assurka']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.24.0,<3.0.0']

entry_points = \
{'pytest11': ['assurka = pytest_assurka.plugin']}

setup_kwargs = {
    'name': 'pytest-assurka',
    'version': '0.2.6',
    'description': 'A pytest plugin for Assurka Studio',
    'long_description': "# pytest-assurka: A pytest plugin for Assurka Studio\n\n\n# Pre-Installation\n\nThe api requests use the `requests` package and this may need to be installed first.\n\n```\npip install requests\n```\n\n# Installation\n\npip install the package to your project's virtual environment. Directly from plugin folder:\n\n\n```bash\npip install -e .\n```\n\nor pip install it from Pypi:\n```bash\npip install pytest-assurka\n```\n\nActivate the plugin with the pytest cli with the command:\n\n```bash\npytest --assurka-projectId={projectId} --assurka-secret={secret} --assurka-testPlanId={testPlanId}\n```\n\nYou can get the above keys from Assurka Studio https://studio.assurka.io",
    'author': 'Assurka Limited',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/assurka-io/pytest-assurka',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
